{# https://github.com/jupyter/nbconvert/blob/5.x/nbconvert/templates/markdown.tpl #}
{%- extends "markdown/index.md.j2" -%}

{% block data_svg %}
{%- if "image/svg+xml" in output.metadata.get("filenames", {}) %}
super()
{%- else %}
{{ output.data['image/svg+xml'] }}
{%- endif %}
{% endblock data_svg %}

{% block data_png %}
{%- if 'image/png' in output.metadata.get('filenames', {}) %}
super()
{%- else %}
<img src="data:image/png;base64,{{ output.data['image/png'] }}">
{%- endif %}
{%- endblock data_png %}

{% block data_jpg %}
{%- if 'image/jpeg' in output.metadata.get('filenames', {}) %}
super()
{%- else %}
<img src="data:image/jpeg;base64,{{ output.data['image/jpeg'] }}"
{%- endif %}
{%- endblock data_jpg %}