{
    "seminar": {
        "group": {{ group.name | jsonify }},
        "semester": {{ group.semester | jsonify }},
        "title": {{ meeting.title | jsonify }},
        "authors": {{ meeting.authors | jsonify }},
        "abstract": {{ meeting.abstract | jsonify }},
        "date": {{ meeting.date.isoformat() | jsonify }},
        "tags": {{ meeting.tags | jsonify }}
    }
}